@@ -5,10 +5,11 @@
 # Author    : Sebastien Pierre                            <sebastien@ffctn.com>
 # Author    : Thierry Stiegler   (gentoo port)     <thierry.stiegler@gmail.com>
 # Author    : Jim McCoy (distro checks and rpm port)      <jim.mccoy@gmail.com>
+# Author    : Warren Moore (zypper package)               <warren@wamonite.com>
 # License   : Revised BSD License
 # -----------------------------------------------------------------------------
 # Creation  : 26-Apr-2010
-# Last mod  : 11-Sep-2012
+# Last mod  : 19-Nov-2012
 # -----------------------------------------------------------------------------
 
 """
@@ -39,10 +40,10 @@ See also:
 """
 
 from __future__ import with_statement
-import base64, bz2, hashlib, os, random, sys, re, string, tempfile, subprocess, types, functools, StringIO
+import base64, zlib, hashlib, os, re, string, tempfile, subprocess, types, functools, StringIO
 import fabric, fabric.api, fabric.operations, fabric.context_managers
 
-VERSION         = "0.4.0"
+VERSION         = "0.4.3"
 RE_SPACES       = re.compile("[\s\t]+")
 MAC_EOL         = "\n"
 UNIX_EOL        = "\n"
@@ -53,12 +54,12 @@ SUDO_PASSWORD   = "CUISINE_SUDO_PASSWORD"
 OPTION_PACKAGE  = "CUISINE_OPTION_PACKAGE"
 OPTION_PYTHON_PACKAGE  = "CUISINE_OPTION_PYTHON_PACKAGE"
 AVAILABLE_OPTIONS = dict(
-	package=["apt", "yum"],
-    python_package=["easy_install","pip"]
+	package=["apt", "yum", "zypper"],
+	python_package=["easy_install","pip"]
 )
 DEFAULT_OPTIONS = dict(
 	package="apt",
-    python_package="pip"
+	python_package="pip"
 )
 
 
@@ -78,14 +79,14 @@ class __mode_switcher(object):
 	MODE_VALUE = True
 	MODE_KEY   = None
 
-	def __init__( self ):
-		self.oldMode = fabric.api.env.get(self.MODE_KEY)
-		fabric.api.env[self.MODE_KEY] = self.MODE_VALUE
+	def __init__( self, value=None ):
+		self.oldMode                  = fabric.api.env.get(self.MODE_KEY)
+		fabric.api.env[self.MODE_KEY] = self.MODE_VALUE if value is None else value
 
 	def __enter__(self):
 		pass
 
-	def __exit__(self, *args, **kwargs):
+	def __exit__(self, type, value, traceback):
 		if self.oldMode is None:
 			del fabric.api.env[self.MODE_KEY]
 		else:
@@ -116,7 +117,7 @@ class mode_sudo(__mode_switcher):
 
 def mode(key):
 	"""Queries the given Cuisine mode (ie. MODE_LOCAL, MODE_SUDO)"""
-	return fabric.api.env.get(key)
+	return fabric.api.env.get(key, False)
 
 def is_local():  return mode(MODE_LOCAL)
 def is_remote(): return not mode(MODE_LOCAL)
@@ -128,20 +129,21 @@ def is_sudo():   return mode(MODE_SUDO)
 #
 # =============================================================================
 
-def select_package( option=None ):
-	"""Selects the type of package subsystem to use (ex:apt or yum)."""
+def select_package( selection=None ):
+	"""Selects the type of package subsystem to use (ex:apt, yum or zypper)."""
 	supported = AVAILABLE_OPTIONS["package"]
-	if not (option is None):
-		assert option in supported, "Option must be one of: %s"  % (supported)
-		fabric.api.env[OPTION_PACKAGE] = option
+	if not (selection is None):
+		assert selection in supported, "Option must be one of: %s"  % (supported)
+		fabric.api.env[OPTION_PACKAGE] = selection
 	return (fabric.api.env[OPTION_PACKAGE], supported)
 
-def select_python_package( option=None ):
-    supported = AVAILABLE_OPTIONS["python_package"]
-    if not (option is None):
-        assert option in supported, "Option must be one of: %s"  % (supported)
-        fabric.api.env[OPTION_PYTHON_PACKAGE] = option
-    return (fabric.api.env[OPTION_PYTHON_PACKAGE], supported)
+def select_python_package( selection=None ):
+	supported = AVAILABLE_OPTIONS["python_package"]
+	if not (selection is None):
+		assert selection in supported, "Option must be one of: %s"  % (supported)
+		fabric.api.env[OPTION_PYTHON_PACKAGE] = selection
+	return (fabric.api.env[OPTION_PYTHON_PACKAGE], supported)
+
 # =============================================================================
 #
 # RUN/SUDO METHODS
@@ -153,7 +155,7 @@ def run_local(command, sudo=False, shell=True, pty=True, combine_stderr=None):
 	Local implementation of fabric.api.run() using subprocess.
 
 	Note: pty option exists for function signature compatibility and is
-	      ignored.
+	ignored.
 	"""
 	if combine_stderr is None: combine_stderr = fabric.api.env.combine_stderr
 	# TODO: Pass the SUDO_PASSWORD variable to the command here
@@ -302,7 +304,7 @@ def text_ensure_line(text, *lines):
 	the end of it."""
 	eol = text_detect_eol(text)
 	res = list(text.split(eol))
-	if res[0] is '' and len(res) is 1:
+	if res[0] == '' and len(res) == 1:
 		res = list()
 	for line in lines:
 		assert line.find(eol) == -1, "No EOL allowed in lines parameter: " + repr(line)
@@ -392,14 +394,15 @@ def file_write(location, content, mode=None, owner=None, group=None, sudo=None,
 	location, optionally setting mode/owner/group."""
 	# FIXME: Big files are never transferred properly!
 	# Gets the content signature and write it to a secure tempfile
-	use_sudo = is_sudo() or sudo
+	use_sudo       = sudo if sudo is not None else is_sudo()
 	sig            = hashlib.sha256(content).hexdigest()
 	fd, local_path = tempfile.mkstemp()
 	os.write(fd, content)
 	# Upload the content if necessary
 	if not file_exists(location) or sig != file_sha256(location):
 		if is_local():
-			run('cp "%s" "%s"'%(local_path,location))
+			with mode_sudo(use_sudo):
+				run('cp "%s" "%s"'%(local_path,location))
 		else:
 			# FIXME: Put is not working properly, I often get stuff like:
 			# Fatal error: sudo() encountered an error (return code 1) while executing 'mv "3dcf7213c3032c812769e7f355e657b2df06b687" "/etc/authbind/byport/80"'
@@ -407,20 +410,27 @@ def file_write(location, content, mode=None, owner=None, group=None, sudo=None,
 			# Hides the output, which is especially important
 			with fabric.context_managers.settings(
 				fabric.api.hide('warnings', 'running', 'stdout'),
-				warn_only=True
+				warn_only=True,
+				**{MODE_SUDO: use_sudo}
 			):
-				# We send the data as BZipped Base64
-				run("echo '%s' | base64 -d | bzcat > \"%s\"" % (base64.b64encode(bz2.compress(content)), location))
+				# See: http://unix.stackexchange.com/questions/22834/how-to-uncompress-zlib-data-in-unix
+				with mode_sudo(use_sudo):
+					result = run("echo '%s' | base64 --decode | openssl zlib -d > \"%s\"" % (base64.b64encode(zlib.compress(content)), location))
+				if result.failed:
+					fabric.api.abort('Encountered error writing the file %s: %s' % (location, result))
+
 	# Remove the local temp file
 	os.close(fd)
 	os.unlink(local_path)
 	# Ensures that the signature matches
 	if check:
-		file_sig = file_sha256(location)
+		with mode_sudo(use_sudo):
+			file_sig = file_sha256(location)
 		assert sig == file_sig, "File content does not matches file: %s, got %s, expects %s" % (location, repr(file_sig), repr(sig))
-	file_attribs(location, mode=mode, owner=owner, group=group)
+	with mode_sudo(use_sudo):
+		file_attribs(location, mode=mode, owner=owner, group=group)
 
-def file_ensure(location, mode=None, owner=None, group=None, recursive=False):
+def file_ensure(location, mode=None, owner=None, group=None):
 	"""Updates the mode/owner/group for the remote file at the given
 	location."""
 	if file_exists(location):
@@ -476,6 +486,7 @@ def file_link(source, destination, symbolic=True, mode=None, owner=None, group=N
 	optionally setting its mode/owner/group."""
 	if file_exists(destination) and (not file_is_link(destination)):
 		raise Exception("Destination already exists and is not a link: %s" % (destination))
+	# FIXME: Should resolve the link first before unlinking
 	if file_is_link(destination):
 		file_unlink(destination)
 	if symbolic:
@@ -489,9 +500,34 @@ def file_sha256(location):
 	# NOTE: In some cases, sudo can output errors in here -- but the errors will
 	# appear before the result, so we simply split and get the last line to
 	# be on the safe side.
-	sig = run('sha256sum "%s" | cut -d" " -f1' % (location)).split("\n")
+	sig = run('shasum -a 256 "%s" | cut -d" " -f1' % (location)).split("\n")
 	return sig[-1].strip()
 
+# =============================================================================
+#
+# PROCESS OPERATIONS
+#
+# =============================================================================
+
+def process_find(name, exact=False):
+	"""Returns the pids of processes with the given name. If exact is `False`
+	it will return the list of all processes that start with the given
+	`name`."""
+	processes = run("ps aux | grep {0}".format(name))
+	res       = []
+	for line in processes.split("\n"):
+		user, pid, cpu, mem, vsz, rss, tty, stat, start, time, command = RE_SPACES.split(line,10)
+		if (exact and command == name) or (not exact and command.startswith(name)):
+			res.append(pid)
+	return res
+
+def process_kill(name, signal=9, exact=False):
+	"""Kills the given processes with the given name. If exact is `False`
+	it will return the list of all processes that start with the given
+	`name`."""
+	for pid in process_find(name, exact):
+		run("kill -s {0} {1}".format(signal, pid))
+
 # =============================================================================
 #
 # DIRECTORY OPERATIONS
@@ -506,6 +542,14 @@ def dir_exists(location):
 	"""Tells if there is a remote directory at the given location."""
 	return run('test -d "%s" && echo OK ; true' % (location)).endswith("OK")
 
+def dir_remove(location, recursive=True):
+	""" Removes a directory """
+	flag = ''
+	if recursive:
+		flag = 'r'
+	if dir_exists(location):
+		return run('rm -%sf %s && echo OK ; true' % (flag, location))
+
 def dir_ensure(location, recursive=False, mode=None, owner=None, group=None):
 	"""Ensures that there is a remote directory at the given location,
 	optionally updating its mode/owner/group.
@@ -515,7 +559,7 @@ def dir_ensure(location, recursive=False, mode=None, owner=None, group=None):
 	if not dir_exists(location):
 		run('mkdir %s "%s" && echo OK ; true' % (recursive and "-p" or "", location))
 	if owner or group or mode:
-		dir_attribs(location, owner=owner, group=group, mode=mode)
+		dir_attribs(location, owner=owner, group=group, mode=mode, recursive=recursive)
 
 # =============================================================================
 #
@@ -531,9 +575,6 @@ def package_upgrade():
 def package_update(package=None):
 	"""Updates the package database (when no argument) or update the package
 	or list of packages given as argument."""
-@dispatch
-def package_update(package=None):
-	"""Upgrade the system."""
 
 @dispatch
 def package_install(package, update=False):
@@ -557,26 +598,23 @@ def package_clean(package=None):
 def repository_ensure_apt(repository):
 	sudo("add-apt-repository " + repository)
 
-def package_upgrade_apt():
-	sudo("apt-get --yes upgrade")
-
 def package_update_apt(package=None):
 	if package == None:
 		sudo("apt-get --yes update")
 	else:
 		if type(package) in (list, tuple):
 			package = " ".join(package)
-		sudo("apt-get --yes upgrade " + package)
+		sudo('DEBIAN_FRONTEND=noninteractive apt-get --yes -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" upgrade ' + package)
 
-def package_upgrade_apt(package=None):
-	sudo("apt-get --yes upgrade")
+def package_upgrade_apt():
+	sudo('DEBIAN_FRONTEND=noninteractive apt-get --yes -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" upgrade')
 
 def package_install_apt(package, update=False):
 	if update:
-		sudo("apt-get --yes update")
+		sudo('DEBIAN_FRONTEND=noninteractive apt-get --yes -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" update')
 	if type(package) in (list, tuple):
 		package = " ".join(package)
-	sudo("apt-get --yes install %s" % (package))
+	sudo("DEBIAN_FRONTEND=noninteractive apt-get --yes install %s" % (package))
 
 def package_ensure_apt(package, update=False):
 	status = run("dpkg-query -W -f='${Status}' %s ; true" % package)
@@ -588,7 +626,7 @@ def package_ensure_apt(package, update=False):
 		return True
 
 def package_clean_apt(package=None):
-    pass
+	pass
 
 # -----------------------------------------------------------------------------
 # YUM PACKAGE (RedHat, CentOS)
@@ -609,9 +647,6 @@ def package_update_yum(package=None):
 			package = " ".join(package)
 		sudo("yum -y upgrade " + package)
 
-def package_upgrade_yum(package=None):
-	sudo("yum -y upgrade")
-
 def package_install_yum(package, update=False):
 	if update:
 		sudo("yum -y update")
@@ -631,6 +666,49 @@ def package_ensure_yum(package, update=False):
 def package_clean_yum(package=None):
 	sudo("yum -y clean all")
 
+# -----------------------------------------------------------------------------
+# ZYPPER PACKAGE (openSUSE)
+# -----------------------------------------------------------------------------
+
+def repository_ensure_zypper(repository):
+	repository_uri = repository
+	if repository[-1] != '/':
+		repository_uri = repository.rpartition("/")[0]
+	status = run("zypper --non-interactive --gpg-auto-import-keys repos -d")
+	if status.find(repository_uri) == -1:
+		sudo("zypper --non-interactive --gpg-auto-import-keys addrepo " + repository)
+		sudo("zypper --non-interactive --gpg-auto-import-keys modifyrepo --refresh " + repository_uri)
+
+def package_upgrade_zypper():
+	sudo("zypper --non-interactive --gpg-auto-import-keys update --type package")
+
+def package_update_zypper(package=None):
+	if package == None:
+		sudo("zypper --non-interactive --gpg-auto-import-keys refresh")
+	else:
+		if type(package) in (list, tuple):
+			package = " ".join(package)
+		sudo("zypper --non-interactive --gpg-auto-import-keys update --type package " + package)
+
+def package_install_zypper(package, update=False):
+	if update:
+		package_update_zypper()
+	if type(package) in (list, tuple):
+		package = " ".join(package)
+	sudo("zypper --non-interactive --gpg-auto-import-keys install --type package --name " + package)
+
+def package_ensure_zypper(package, update=False):
+	status = run("zypper --non-interactive --gpg-auto-import-keys search --type package --installed-only --match-exact %s ; true" % package)
+	if status.find("No packages found.") != -1 or status.find(package) == -1:
+		package_install_zypper(package)
+		return False
+	else:
+		if update:
+			package_update_zypper(package)
+		return True
+
+def package_clean_zypper():
+	sudo("zypper --non-interactive clean")
 
 # =============================================================================
 #
@@ -641,87 +719,87 @@ def package_clean_yum(package=None):
 @dispatch('python_package')
 def python_package_upgrade(package):
 	'''
-    Upgrades the defined python package.
-    '''
+	Upgrades the defined python package.
+	'''
 
 @dispatch('python_package')
 def python_package_install(package=None):
 	'''
-    Installs the given python package/list of python packages.
-    '''
+	Installs the given python package/list of python packages.
+	'''
 
 @dispatch('python_package')
 def python_package_ensure(package):
 	'''
-    Tests if the given python package is installed, and installes it in
+	Tests if the given python package is installed, and installes it in
 	case it's not already there.
-    '''
+	'''
 
 @dispatch('python_package')
 def python_package_remove(package):
 	'''
-    Removes the given python package.
-    '''
+	Removes the given python package.
+	'''
 
 # -----------------------------------------------------------------------------
 # PIP PYTHON PACKAGE MANAGER
 # -----------------------------------------------------------------------------
 
 def python_package_upgrade_pip(package,E=None):
-    '''
-    The "package" argument, defines the name of the package that will be upgraded.
-    The optional argument "E" is equivalent to the "-E" parameter of pip. E is the
-    path to a virtualenv. If provided, it will be added to the pip call.
-    '''
-    if E:
-        E='-E %s' %E
-    else:
-        E=''   
-    run('pip upgrade %s %s' %(E,package))
+	'''
+	The "package" argument, defines the name of the package that will be upgraded.
+	The optional argument "E" is equivalent to the "-E" parameter of pip. E is the
+	path to a virtualenv. If provided, it will be added to the pip call.
+	'''
+	if E:
+		E='-E %s' %E
+	else:
+		E=''   
+	run('pip upgrade %s %s' %(E,package))
 
 def python_package_install_pip(package=None,r=None,pip=None):
-    '''
-    The "package" argument, defines the name of the package that will be installed.
-    The argument "r" referes to the requirements file that will be used by pip and
-    is equivalent to the "-r" parameter of pip.
-    Either "package" or "r" needs to be provided
-    The optional argument "E" is equivalent to the "-E" parameter of pip. E is the
-    path to a virtualenv. If provided, it will be added to the pip call.
-    '''
-    pip=pip or fabric.api.env.env.get('pip','pip')
-    if package:
-        run('%s install %s' %(pip,package))
-    elif r:
-        run('%s install -r %s' %(pip,r))
-    else:
-        raise Exception("Either a package name or the requirements file has to be provided.")
+	'''
+	The "package" argument, defines the name of the package that will be installed.
+	The argument "r" referes to the requirements file that will be used by pip and
+	is equivalent to the "-r" parameter of pip.
+	Either "package" or "r" needs to be provided
+	The optional argument "E" is equivalent to the "-E" parameter of pip. E is the
+	path to a virtualenv. If provided, it will be added to the pip call.
+	'''
+	pip=pip or fabric.api.env.get('pip','pip')
+	if package:
+		run('%s install %s' %(pip,package))
+	elif r:
+		run('%s install -r %s' %(pip,r))
+	else:
+		raise Exception("Either a package name or the requirements file has to be provided.")
 
 def python_package_ensure_pip(package=None,r=None, pip=None):
-    '''
-    The "package" argument, defines the name of the package that will be ensured.
-    The argument "r" referes to the requirements file that will be used by pip and
-    is equivalent to the "-r" parameter of pip.
-    Either "package" or "r" needs to be provided
-    The optional argument "E" is equivalent to the "-E" parameter of pip. E is the
-    path to a virtualenv. If provided, it will be added to the pip call.
-    '''
-    #FIXME: At the moment, I do not know how to check for the existence of a pip package and
-    # I am not sure if this really makes sense, based on the pip built in functionality. 
-    # So I just call the install functions
-    pip=pip or fabric.api.env.env.get('pip','pip')
-    python_package_install_pip(package,r,pip)
+	'''
+	The "package" argument, defines the name of the package that will be ensured.
+	The argument "r" referes to the requirements file that will be used by pip and
+	is equivalent to the "-r" parameter of pip.
+	Either "package" or "r" needs to be provided
+	The optional argument "E" is equivalent to the "-E" parameter of pip. E is the
+	path to a virtualenv. If provided, it will be added to the pip call.
+	'''
+	#FIXME: At the moment, I do not know how to check for the existence of a pip package and
+	# I am not sure if this really makes sense, based on the pip built in functionality. 
+	# So I just call the install functions
+	pip=pip or fabric.api.env.get('pip','pip')
+	python_package_install_pip(package,r,pip)
 
-def python_package_remove_pip(package, E=None):
-    '''
-    The "package" argument, defines the name of the package that will be ensured.
-    The argument "r" referes to the requirements file that will be used by pip and
-    is equivalent to the "-r" parameter of pip.
-    Either "package" or "r" needs to be provided
-    The optional argument "E" is equivalent to the "-E" parameter of pip. E is the
-    path to a virtualenv. If provided, it will be added to the pip call. 
-    '''
-    pip=pip or fabric.api.env.env.get('pip','pip')
-    return run('%s uninstall %s' %(pip,package))
+def python_package_remove_pip(package, E=None, pip=None):
+	'''
+	The "package" argument, defines the name of the package that will be ensured.
+	The argument "r" referes to the requirements file that will be used by pip and
+	is equivalent to the "-r" parameter of pip.
+	Either "package" or "r" needs to be provided
+	The optional argument "E" is equivalent to the "-E" parameter of pip. E is the
+	path to a virtualenv. If provided, it will be added to the pip call. 
+	'''
+	pip=pip or fabric.api.env.get('pip','pip')
+	return run('%s uninstall %s' %(pip,package))
 
 
 # -----------------------------------------------------------------------------
@@ -729,32 +807,32 @@ def python_package_remove_pip(package, E=None):
 # -----------------------------------------------------------------------------
 
 def python_package_upgrade_easy_install(package):
-    '''
-    The "package" argument, defines the name of the package that will be upgraded.
-    '''
-    run('easy_install --upgrade %s' %package)
+	'''
+	The "package" argument, defines the name of the package that will be upgraded.
+	'''
+	run('easy_install --upgrade %s' %package)
 
 def python_package_install_easy_install(package):
-    '''
-    The "package" argument, defines the name of the package that will be installed.
-    '''
-    sudo('easy_install %s' %package)
+	'''
+	The "package" argument, defines the name of the package that will be installed.
+	'''
+	sudo('easy_install %s' %package)
 
 def python_package_ensure_easy_install(package):
-    '''
-    The "package" argument, defines the name of the package that will be ensured.
-    '''
-    #FIXME: At the moment, I do not know how to check for the existence of a py package and
-    # I am not sure if this really makes sense, based on the easy_install built in functionality. 
-    # So I just call the install functions
-    python_package_install_easy_install(package)
+	'''
+	The "package" argument, defines the name of the package that will be ensured.
+	'''
+	#FIXME: At the moment, I do not know how to check for the existence of a py package and
+	# I am not sure if this really makes sense, based on the easy_install built in functionality. 
+	# So I just call the install functions
+	python_package_install_easy_install(package)
 
 def python_package_remove_easy_install(package):
-    '''
-    The "package" argument, defines the name of the package that will be removed.
-    '''
-    #FIXME: this will not remove egg file etc.
-    run('easy_install -m %s' %package) 
+	'''
+	The "package" argument, defines the name of the package that will be removed.
+	'''
+	#FIXME: this will not remove egg file etc.
+	run('easy_install -m %s' %package) 
 
 # =============================================================================
 #
@@ -1011,7 +1089,10 @@ def locale_ensure(locale):
 		sudo("dpkg-reconfigure locales")
 
 # Sets up the default options so that @dispatch'ed functions work
-for option, value in DEFAULT_OPTIONS.items():
-	eval("select_" + option)(value)
+def _init():
+	for option, value in DEFAULT_OPTIONS.items():
+		eval("select_" + option)(value)
+
+_init()
 
 # EOF - vim: ts=4 sw=4 noet
